@@ -1,4 +1,3 @@
-from lightning_app.components.auto_scaler import AutoScaler
 from lightning_app.components.database.client import DatabaseClient
 from lightning_app.components.database.server import Database
 from lightning_app.components.multi_node import (
@@ -9,14 +8,17 @@ from lightning_app.components.multi_node import (
 )
 from lightning_app.components.python.popen import PopenPythonScript
 from lightning_app.components.python.tracer import Code, TracerPythonScript
-from lightning_app.components.serve.gradio import ServeGradio
-from lightning_app.components.serve.python_server import Image, Number, PythonServer
+from lightning_app.components.serve.auto_scaler import AutoScaler
+from lightning_app.components.serve.cold_start_proxy import ColdStartProxy
+from lightning_app.components.serve.gradio_server import ServeGradio
+from lightning_app.components.serve.python_server import Category, Image, Number, PythonServer, Text
 from lightning_app.components.serve.serve import ModelInferenceAPI
 from lightning_app.components.serve.streamlit import ServeStreamlit
 from lightning_app.components.training import LightningTrainerScript, PyTorchLightningScriptRunner
 
 __all__ = [
     "AutoScaler",
+    "ColdStartProxy",
     "DatabaseClient",
     "Database",
     "PopenPythonScript",
@@ -28,6 +30,8 @@ __all__ = [
     "PythonServer",
     "Image",
     "Number",
+    "Category",
+    "Text",
     "MultiNode",
     "LiteMultiNode",
     "LightningTrainerScript",
