@@ -1,11 +1,26 @@
+# Copyright The Lightning AI team.
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#     http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+
 import multiprocessing
 from typing import List, Optional
 
 import lightning_app
+from lightning_app.core import constants
 from lightning_app.core.queues import QueuingSystem
 from lightning_app.runners.backends.backend import Backend, WorkManager
 from lightning_app.utilities.enum import WorkStageStatus
-from lightning_app.utilities.network import _check_service_url_is_ready
+from lightning_app.utilities.network import _check_service_url_is_ready, find_free_network_port
 from lightning_app.utilities.port import disable_port, enable_port
 from lightning_app.utilities.proxies import ProxyWorkRun, WorkRunner
 
@@ -62,6 +77,12 @@ class MultiProcessingBackend(Backend):
         super().__init__(entrypoint_file=entrypoint_file, queues=QueuingSystem.MULTIPROCESS, queue_id="0")
 
     def create_work(self, app, work) -> None:
+        if constants.LIGHTNING_CLOUDSPACE_HOST is not None:
+            # Override the port if set by the user
+            work._port = find_free_network_port()
+            work._host = "0.0.0.0"
+            work._future_url = f"https://{work.port}-{constants.LIGHTNING_CLOUDSPACE_HOST}"
+
         app.processes[work.name] = MultiProcessWorkManager(app, work)
         app.processes[work.name].start()
         self.resolve_url(app)
